package dev.fyloz.trial.colorrecipesexplorer.repository

import dev.fyloz.trial.colorrecipesexplorer.model.Material
import dev.fyloz.trial.colorrecipesexplorer.model.MaterialType
import org.springframework.data.jpa.repository.Query
import org.springframework.stereotype.Repository

@Repository
interface MaterialRepository : NamedJpaRepository<Material> {
    /** Checks if one or more materials have the given [materialType]. */
    fun existsByMaterialType(materialType: MaterialType): Boolean

    /** Gets all the materials with the given [materialType]. */
    fun findAllByMaterialType(materialType: MaterialType): Collection<Material>

    @Query(
        """
        select case when(count(mm.id) + count(mt.id) > 0) then false else true end
        from Material m
        left join MixMaterial mm on m.id = mm.material.id
        left join MixType mt on m.id = mt.material.id
        where m.id = :id
    """
    )
    fun canBeDeleted(id: Long): Boolean
}
